import { persistQueryClient } from '@tanstack/query-persist-client-core';
export * from '@tanstack/query-persist-client-core';
import { createComponent } from 'solid-js/web';
import { createSignal, createComputed, onCleanup } from 'solid-js';
import { QueryClientProvider, IsRestoringProvider } from '@tanstack/solid-query';

// src/index.ts
var PersistQueryClientProvider = (props) => {
  const [isRestoring, setIsRestoring] = createSignal(true);
  let unsub;
  createComputed((cleanup) => {
    cleanup?.();
    let isStale = false;
    setIsRestoring(true);
    const [unsubscribe, promise] = persistQueryClient({
      ...props.persistOptions,
      queryClient: props.client
    });
    promise.then(async () => {
      if (isStale)
        return;
      try {
        await props.onSuccess?.();
      } finally {
        setIsRestoring(false);
      }
    });
    unsub = () => {
      isStale = true;
      unsubscribe();
    };
    return unsub;
  });
  onCleanup(() => unsub?.());
  return createComponent(QueryClientProvider, {
    get client() {
      return props.client;
    },
    get children() {
      return createComponent(IsRestoringProvider, {
        value: isRestoring,
        get children() {
          return props.children;
        }
      });
    }
  });
};

export { PersistQueryClientProvider };
